@@ -16,19 +16,21 @@
 #
 # Copyright (c) 2011,2012 peo3 <peo314159265@gmail.com>
 
-from __future__ import with_statement
-import os, os.path
+import os
+import os.path
 import re
 import struct
+import errno
 
 from cgutils import host
-from cgutils import linux
+from cgutils import process
+import fileops
 
-def readfile(filepath):
-    with open(filepath) as f:
-        return f.read()
 
 class SubsystemStatus(dict):
+    __RE = '^(?P<name>\w+)\s+(?P<hier>\d+)\s+(?P<n>\d+)\s+(?P<enabled>[01])'
+    _RE_CGROUPS = re.compile(__RE)
+
     def __init__(self):
         dict.__init__(self)
         self.paths = {}
@@ -48,11 +50,10 @@ class SubsystemStatus(dict):
         freezer	0	1	1
         net_cls	0	1	1
         """
-        p = re.compile('^(?P<name>\w+)\s+(?P<hier>\d+)\s+(?P<n>\d+)\s+(?P<enabled>[01])')
-        lines = readfile('/proc/cgroups').split('\n')
-        for line in lines:
-            m = p.match(line)
-            if m is None: continue
+        for line in fileops.readlines('/proc/cgroups'):
+            m = self._RE_CGROUPS.match(line)
+            if m is None:
+                continue
 
             name = m.group('name')
             hierarchy = int(m.group('hier'))
@@ -72,15 +73,15 @@ class SubsystemStatus(dict):
         """Parse /proc/mounts"""
 
         """
-        cgroup /dev/cgroup/cpu cgroup rw,relatime,cpuacct,cpu,release_agent=/usr/local/sbin/cgroup_clean 0 0
-        cgroup /dev/cgroup/memory cgroup rw,relatime,memory 0 0
-        cgroup /dev/cgroup/blkio cgroup rw,relatime,blkio 0 0
-        cgroup /dev/cgroup/freezer cgroup rw,relatime,freezer 0 0
+        cgroup /cgroup/cpu cgroup rw,relatime,cpuacct,cpu,release_agent=/sbin/cgroup_clean 0 0
+        cgroup /cgroup/memory cgroup rw,relatime,memory 0 0
+        cgroup /cgroup/blkio cgroup rw,relatime,blkio 0 0
+        cgroup /cgroup/freezer cgroup rw,relatime,freezer 0 0
         """
 
-        lines = readfile('/proc/mounts').split('\n')
-        for line in lines:
-            if 'cgroup' not in line: continue
+        for line in fileops.readlines('/proc/mounts'):
+            if 'cgroup' not in line:
+                continue
 
             items = line.split(' ')
             path = items[1]
@@ -118,7 +119,7 @@ class SubsystemStatus(dict):
 
     def get_available(self):
         return [name for name in self.keys()
-                if self[name]['enabled'] ]
+                if self[name]['enabled']]
 
     def get_enabled(self):
         return self.paths.keys()
@@ -126,45 +127,31 @@ class SubsystemStatus(dict):
     def get_path(self, subsys):
         return self.paths[subsys]
 
-"""
-100
-103
-234
-"""
-class SimpleList(dict):
+
+class SimpleList(list):
     @staticmethod
-    def parse(path):
+    def parse(content):
         ret = []
-        for line in readfile(path).split('\n')[:-1]:
+        for line in content.split('\n')[:-1]:
             ret.append(long(line))
         return ret
 
-"""
-user 2978976
-system 1037760
-"""
+
 class SimpleStat(dict):
     @staticmethod
-    def parse(path):
+    def parse(content):
         ret = {}
-        for line in readfile(path).split('\n')[:-1]:
+        for line in content.split('\n')[:-1]:
             name, val = line.split(' ')
             ret[name] = long(val)
         return ret
 
-"""
-8:0 Read 72650752
-8:0 Write 28090368
-8:0 Sync 28090368
-8:0 Async 72650752
-8:0 Total 100741120
-Total 100741120
-"""
+
 class BlkioStat(dict):
     @staticmethod
-    def parse(path):
+    def parse(content):
         ret = {}
-        for line in readfile(path).split('\n')[:-1]:
+        for line in content.split('\n')[:-1]:
             if line.count(' ') == 2:
                 dev, type, val = line.split(' ')
                 if dev not in ret:
@@ -177,42 +164,18 @@ class BlkioStat(dict):
                 raise EnvironmentError(line)
         return ret
 
-"""
-a *:* rwm
 
-c 136:* rwm
-c 1:3 rwm
-c 1:7 rwm
-c 1:5 rwm
-c 1:8 rwm
-c 1:9 rwm
-c 5:2 rwm
-c 10:232 rwm
-c 254:0 rwm
-c 10:228 rwm
-c 10:200 rwm
-c 251:0 rwm
-c 251:1 rwm
-c 251:2 rwm
-c 251:3 rwm
-c 251:4 rwm
-"""
 class DevicesStat(list):
     @staticmethod
-    def parse(path):
-        return readfile(path).split('\n')[:-1]
+    def parse(content):
+        return [v for v in content.split('\n')[:-1] if v]
+
 
-"""
-total=83920 N0=83920
-file=63452 N0=63452
-anon=20468 N0=20468
-unevictable=0 N0=0
-"""
 class NumaStat(dict):
     @staticmethod
-    def parse(path):
+    def parse(content):
         ret = {}
-        lines = readfile(path).split('\n')[:-1]
+        lines = content.split('\n')[:-1]
         for line in lines:
             item = {}
             entries = line.split(' ')
@@ -224,24 +187,22 @@ class NumaStat(dict):
             ret[name] = item
         return ret
 
-"""
-# Two CPUs
-836842800783 656015556351 
-"""
+
 class PercpuStat(dict):
     @staticmethod
-    def parse(path):
+    def parse(content):
         ret = {}
-        line = readfile(path).split('\n')[0]
+        line = content.split('\n')[0]
         stats = line.split(' ')
         # A line may end with a redundant space
-        stats = [ stat for stat in stats if stat != '' ]
+        stats = [stat for stat in stats if stat != '']
         i = 0
         for stat in stats:
             ret[i] = long(stat)
             i += 1
         return ret
 
+
 #
 # The base class of subsystems
 #
@@ -254,12 +215,16 @@ class Subsystem(object):
     def __init__(self):
         self.name = self.NAME
 
+    def get_init_parameters(self, parent_configs):
+        return {}
+
+
 #
 # Classes of each subsystem
 #
 class SubsystemCpu(Subsystem):
     NAME = 'cpu'
-    _path_rt_period  = '/proc/sys/kernel/sched_rt_period_us'
+    _path_rt_period = '/proc/sys/kernel/sched_rt_period_us'
     _path_rt_runtime = '/proc/sys/kernel/sched_rt_runtime_us'
     STATS = {
         'stat': SimpleStat,
@@ -267,12 +232,12 @@ class SubsystemCpu(Subsystem):
     CONFIGS = {
         'shares':        1024,
         # Are the default values correct?
-        'rt_period_us':  long(readfile(_path_rt_period)),
-        'rt_runtime_us': long(readfile(_path_rt_runtime)),
+        'rt_period_us':  long(fileops.read(_path_rt_period)),
+        'rt_runtime_us': long(fileops.read(_path_rt_runtime)),
         'cfs_period_us': 100000,
         'cfs_quota_us': -1,
     }
-    CONTROLS = {}
+
 
 class SubsystemCpuacct(Subsystem):
     NAME = 'cpuacct'
@@ -281,8 +246,7 @@ class SubsystemCpuacct(Subsystem):
         'stat': SimpleStat,
         'usage_percpu': PercpuStat,
     }
-    CONFIGS = {}
-    CONTROLS = {}
+
 
 class SubsystemCpuset(Subsystem):
     NAME = 'cpuset'
@@ -304,7 +268,13 @@ class SubsystemCpuset(Subsystem):
         'sched_load_balance': 1,
         'sched_relax_domain_level': -1,
     }
-    CONTROLS = {}
+
+    def get_init_parameters(self, parent_configs):
+        params = {}
+        params['cpus'] = parent_configs['cpus']
+        params['mems'] = parent_configs['mems']
+        return params
+
 
 class SubsystemMemory(Subsystem):
     NAME = 'memory'
@@ -321,12 +291,12 @@ class SubsystemMemory(Subsystem):
         'kmem.tcp.max_usage_in_bytes': long,
         'kmem.tcp.usage_in_bytes': long,
     }
-    MAX_ULONGLONG = 2**63-1
+    MAX_ULONGLONG = 2 ** 63 - 1
     CONFIGS = {
         'limit_in_bytes': MAX_ULONGLONG,
         'memsw.limit_in_bytes': MAX_ULONGLONG,
         'move_charge_at_immigrate': 0,
-        'oom_control': SimpleStat({'oom_kill_disable':0, 'under_oom':0}),
+        'oom_control': SimpleStat({'oom_kill_disable': 0, 'under_oom': 0}),
         'soft_limit_in_bytes': MAX_ULONGLONG,
         'swappiness': 60,
         'use_hierarchy': 0,
@@ -336,6 +306,7 @@ class SubsystemMemory(Subsystem):
         'force_empty': None,
     }
 
+
 class SubsystemBlkio(Subsystem):
     NAME = 'blkio'
     STATS = {
@@ -362,25 +333,23 @@ class SubsystemBlkio(Subsystem):
         'reset_stats': None,
     }
 
+
 class SubsystemFreezer(Subsystem):
     NAME = 'freezer'
     STATS = {
         'state': str,
     }
-    CONFIGS = {}
-    CONTROLS = {}
+
 
 class SubsystemNetCls(Subsystem):
     NAME = 'net_cls'
-    STATS = {}
     CONFIGS = {
         'classid': 0,
     }
-    CONTROLS = {}
+
 
 class SubsystemDevices(Subsystem):
     NAME = 'devices'
-    STATS = {}
     CONFIGS = {
         'list': DevicesStat(['a *:* rwm']),
     }
@@ -389,49 +358,86 @@ class SubsystemDevices(Subsystem):
         'deny': None,
     }
 
+
+class SubsystemNetPrio(Subsystem):
+    NAME = 'net_prio'
+    STATS = {
+        'prioidx': long,
+    }
+    __ifs = os.listdir('/sys/class/net')
+    CONFIGS = {
+        'ifpriomap': SimpleStat(zip(__ifs, [0] * len(__ifs))),
+    }
+
+
 class SubsystemName(Subsystem):
     NAME = 'name'
-    STATS = {}
-    CONFIGS = {}
-    CONTROLS = {}
 
     def __init__(self, name):
         Subsystem.__init__(self)
         self.name = name
 
+
 _subsystem_name2class = {
-    'cpu':SubsystemCpu,
-    'cpuacct':SubsystemCpuacct,
-    'cpuset':SubsystemCpuset,
-    'memory':SubsystemMemory,
-    'blkio':SubsystemBlkio,
-    'freezer':SubsystemFreezer,
-    'net_cls':SubsystemNetCls,
-    'devices':SubsystemDevices,
+    'cpu': SubsystemCpu,
+    'cpuacct': SubsystemCpuacct,
+    'cpuset': SubsystemCpuset,
+    'memory': SubsystemMemory,
+    'blkio': SubsystemBlkio,
+    'freezer': SubsystemFreezer,
+    'net_cls': SubsystemNetCls,
+    'devices': SubsystemDevices,
+    'net_prio': SubsystemNetPrio,
 }
+
+
 def _get_subsystem(name):
     if 'name=' in name:
         return SubsystemName(name)
     return _subsystem_name2class[name]()
 
-class CGroup(object):
-    STATS = {
+
+class NoSuchControlFileError(StandardError):
+    pass
+
+
+class IsRootGroupError(StandardError):
+    pass
+
+
+class CGroup:
+    """
+    This class represents a control group in a cgroup hierarchy.
+
+    An instance belongs to a paticluar subsystem, thus, only one
+    file path is determined to the instance.
+
+    An instance has control files of both a cgroup and a subsystem
+    which the instance belongs to. An instance can have child
+    cgroups under the cgroup directory of the instace.
+
+    The class categorises control files into three groups: configs
+    include configurable files, stats include statistics of the
+    group, and controls include special configuration files which
+    are normally write-only.
+    """
+    _STATS = {
         'tasks': SimpleList,
         'cgroup.procs': SimpleList,
     }
-    CONFIGS = {
+    _CONFIGS = {
         'release_agent': '',
         # XXX: the default value is actually inherited from a parent
         'notify_on_release': 0,
         'cgroup.clone_children': 0,
     }
-    CONTROLS = {
+    _CONTROLS = {
         'cgroup.event_control': None,
     }
-    PARSERS = {
-        int:  lambda path: int(readfile(path)),
-        long: lambda path: long(readfile(path)),
-        str:  lambda path: readfile(path).strip(),
+    _PARSERS = {
+        int: lambda content: int(content),
+        long: lambda content: long(content),
+        str: lambda content: content.strip(),
         SimpleList: SimpleList.parse,
         SimpleStat: SimpleStat.parse,
         BlkioStat: BlkioStat.parse,
@@ -449,9 +455,10 @@ class CGroup(object):
                 return rec(rest) + 1
         return rec(path)
 
-    def __init__(self, subsystem, fullpath, filters=list()):
+    def __init__(self, subsystem, fullpath, parent=None, filters=list()):
         self.subsystem = subsystem
         self.fullpath = fullpath
+        self.parent = parent
         self.filters = filters
 
         status = SubsystemStatus()
@@ -461,39 +468,30 @@ class CGroup(object):
 
         if self.path == '/':
             self.depth = 0
-            self.fullname = self.name = '<root>'
+            self.fullname = self.name = '/'
         else:
             self.depth = self._calc_depth(self.path)
             self.name = os.path.basename(self.path)
             self.fullname = self.path[1:]
 
+        if self.parent is None and self.depth != 0:
+            # XXX: We should do out of the class?
+            self.parent = get_cgroup(os.path.dirname(self.fullpath))
+
         self.paths = {}
-        for file in self.STATS.keys() + self.CONFIGS.keys() + self.CONTROLS.keys():
+        for file in self._STATS.keys() + self._CONFIGS.keys() + self._CONTROLS.keys():
             self.paths[file] = os.path.join(self.fullpath, file)
-        for file in subsystem.STATS.keys() + subsystem.CONFIGS.keys() + \
-                    subsystem.CONTROLS.keys():
-            self.paths[file] = os.path.join(self.fullpath, subsystem.name+'.'+file)
+        for file in subsystem.STATS.keys() + subsystem.CONFIGS.keys() + subsystem.CONTROLS.keys():
+            self.paths[file] = os.path.join(self.fullpath, subsystem.name + '.' + file)
 
         self.configs = {}
+        self.configs.update(self._CONFIGS)
+        self.configs.update(subsystem.CONFIGS)
         self.stats = {}
-
+        self.stats.update(self._STATS)
+        self.stats.update(subsystem.STATS)
         if self.filters:
-            _configs = {}
-            _configs.update(self.CONFIGS)
-            _configs.update(subsystem.CONFIGS)
-            _stats = {}
-            _stats.update(self.STATS)
-            _stats.update(subsystem.STATS)
-            for f in self.filters:
-                if f in _configs:
-                    self.configs[f] = _configs[f]
-                elif f in _stats:
-                    self.stats[f] = _stats[f]
-        else:
-            self.configs.update(self.CONFIGS)
-            self.configs.update(subsystem.CONFIGS)
-            self.stats.update(self.STATS)
-            self.stats.update(subsystem.STATS)
+            self.apply_filters(filters)
 
         self.childs = []
         self.pids = []
@@ -504,36 +502,147 @@ class CGroup(object):
     def __str__(self):
         return "<CGroup: %s (%s)>" % (self.fullname, self.subsystem.name)
 
+    def __hash__(self):
+        return hash((self.fullname, self.subsystem.name))
+
+    def __eq__(self, obj):
+        return self.fullname == obj.fullname and self.subsystem.name == obj.subsystem.name
+
+    def apply_filters(self, filters):
+        """
+        It applies a specified filters. The filters are used to reduce the control groups
+        which are accessed by get_confgs, get_stats, and get_defaults methods.
+        """
+        _configs = self.configs
+        _stats = self.stats
+        self.configs = {}
+        self.stats = {}
+        for f in filters:
+            if f in _configs:
+                self.configs[f] = _configs[f]
+            elif f in _stats:
+                self.stats[f] = _stats[f]
+            else:
+                raise NoSuchControlFileError("%s for %s" % (f, self.subsystem.name))
+
     def get_configs(self):
+        """
+        It returns a name and a current value pairs of control files
+        which are categorised in the configs group.
+        """
         configs = {}
         for name, default in self.configs.iteritems():
             cls = default.__class__
             path = self.paths[name]
             if os.path.exists(path):
-                configs[name] = self.PARSERS[cls](path)
+                try:
+                    configs[name] = self._PARSERS[cls](fileops.read(path))
+                except IOError, e:
+                    if e.errno == errno.EOPNOTSUPP:
+                        # Since 3.5 memory.memsw.* are always created even if disabled.
+                        # If disabled we will get EOPNOTSUPP when read or write them.
+                        # See commit af36f906c0f4c2ffa0482ecdf856a33dc88ae8c5 of the kernel.
+                        pass
+                    else:
+                        raise
         return configs
 
     def get_default_configs(self):
+        """
+        It is similar to get_configs but it returns default values
+        instead of current values.
+        """
         return self.configs.copy()
 
     def get_stats(self):
+        """
+        It returns a name and a value pairs of control files
+        which are categorised in the stats group.
+        """
         stats = {}
         for name, cls in self.stats.iteritems():
             path = self.paths[name]
             if os.path.exists(path):
-                stats[name] = self.PARSERS[cls](path)
+                try:
+                    stats[name] = self._PARSERS[cls](fileops.read(path))
+                except IOError, e:
+                    if e.errno == errno.EOPNOTSUPP:
+                        # Since 3.5 memory.memsw.* are always created even if disabled.
+                        # If disabled we will get EOPNOTSUPP when read or write them.
+                        # See commit af36f906c0f4c2ffa0482ecdf856a33dc88ae8c5 of the kernel.
+                        pass
+                    else:
+                        raise
         return stats
 
     def update(self):
-        pids = readfile(self.paths['cgroup.procs']).rstrip('\n').split('\n')
+        """It updates process information of the cgroup."""
+        pids = fileops.readlines(self.paths['cgroup.procs'])
         self.pids = [int(pid) for pid in pids if pid != '']
         self.n_procs = len(pids)
 
-class EventListener(object):
-    def __init__(self, cgroup, target_path):
-        self.cgroup = cgroup
+    def set_config(self, name, value):
+        path = os.path.join(self.fullpath, self.subsystem.name + '.' + name)
+        fileops.write(path, str(value))
 
-        # To keep the files open
+    def mkdir(self, name, set_initparams=True):
+        new_path = os.path.join(self.fullpath, name)
+        fileops.mkdir(new_path)
+        new = get_cgroup(new_path)
+        if set_initparams:
+            params = self.subsystem.get_init_parameters(self.get_configs())
+            for filename, value in params.iteritems():
+                new.set_config(filename, value)
+        return new
+
+    def rmdir(self, target=None):
+        if self.depth == 0:
+            raise IsRootGroupError("%s is a root cgroup" % self.fullpath)
+
+        if target:
+            if not isinstance(target, CGroup):
+                raise TypeError('Not a CGroup instance')
+        else:
+            target = self.parent
+
+        self.update()
+
+        for pid in self.pids:
+            target.attach(pid)
+        fileops.rmdir(self.fullpath)
+
+    def attach(self, pid):
+        if not process.exists(pid):
+            raise EnvironmentError("Process %d not exists" % pid)
+
+        fileops.write(self.paths['tasks'], str(pid))
+
+
+class EventListener:
+    """
+    It enable us to use event notification feature of control groups.
+
+    To use this feature, we have to specify a cgroup
+    and a target control file of the cgroup.
+    """
+    SUPPORTED_FILES = [
+        'memory.usage_in_bytes',
+        'memory.oom_control',
+        'memory.memsw.usage_in_bytes',
+    ]
+
+    def __init__(self, cgroup, target_name):
+        from cgutils import linux
+
+        self.cgroup = cgroup
+        self.target_name = target_name
+
+        if target_name not in self.SUPPORTED_FILES:
+            raise EnvironmentError("%s is not supported by the kernel" % target_name)
+
+        target_path = os.path.join(cgroup.fullpath, target_name)
+
+        # To keep the files open, set them in instance variables
         self.target_file = open(target_path)
         self.target_fd = self.target_file.fileno()
 
@@ -543,14 +652,28 @@ class EventListener(object):
 
         self.event_fd = linux.eventfd(0, 0)
 
-    def set_threshold(self, threshold):
-        line = "%d %d %d\0" % (self.event_fd, self.target_fd, threshold)
+    def register(self, arguments=list()):
+        """
+        Register a target file with arguments (if required) to a event_control file
+        which we want to be notified events.
+        """
+        target_name = self.target_name
+        if target_name in ['memory.usage_in_bytes', 'memory.memsw.usage_in_bytes']:
+            threshold = arguments[0]
+            line = "%d %d %d\0" % (self.event_fd, self.target_fd, long(threshold))
+        else:
+            line = "%d %d\0" % (self.event_fd, self.target_fd)
         os.write(self.ec_fd, line)
 
     def wait(self):
-        ret = os.read(self.event_fd, 64/8)
+        """
+        It returns when an event which we have configured by set_threshold happens.
+        Note that it blocks until then.
+        """
+        ret = os.read(self.event_fd, 64 / 8)
         return struct.unpack('Q', ret)
 
+
 def _scan_cgroups_recursive(subsystem, fullpath, mount_point, filters):
     cgroup = CGroup(subsystem, fullpath, filters)
 
@@ -559,17 +682,25 @@ def _scan_cgroups_recursive(subsystem, fullpath, mount_point, filters):
         child_fullpath = os.path.join(fullpath, _file)
         if os.path.isdir(child_fullpath):
             child = _scan_cgroups_recursive(subsystem, child_fullpath,
-                                           mount_point, filters)
+                                            mount_point, filters)
             _childs.append(child)
     cgroup.childs.extend(_childs)
     return cgroup
 
-"""
-  Public APIs
-"""
-class NoSuchSubsystemError(StandardError): pass
+
+#
+#  Public APIs
+#
+class NoSuchSubsystemError(StandardError):
+    pass
+
 
 def scan_cgroups(subsys_name, filters=list()):
+    """
+    It returns a control group hierarchy which belong to the subsys_name.
+    When collecting cgroups, filters are applied to the cgroups. See pydoc
+    of apply_filters method of CGroup for more information about the filters.
+    """
     status = SubsystemStatus()
     if subsys_name not in status.get_all():
         raise NoSuchSubsystemError("No such subsystem found: " + subsys_name)
@@ -584,16 +715,28 @@ def scan_cgroups(subsys_name, filters=list()):
     mount_point = status.get_path(subsys_name)
     return _scan_cgroups_recursive(subsystem, mount_point, mount_point, filters)
 
+
 def walk_cgroups(cgroup, action, opaque):
+    """
+    The function applies the action function with the opaque object
+    to each control group under the cgroup recursively.
+    """
     action(cgroup, opaque)
     for child in cgroup.childs:
         walk_cgroups(child, action, opaque)
 
+
 def get_cgroup(fullpath):
+    """
+    It returns a CGroup object which is pointed by the fullpath.
+    """
+    # Canonicalize symbolic links
+    fullpath = os.path.realpath(fullpath)
+
     status = SubsystemStatus()
     name = None
-    for name, in status.paths.iteritems():
-        if name in fullpath:
+    for name, path in status.paths.iteritems():
+        if path in fullpath:
             break
     else:
         raise StandardError('Invalid path: ' + fullpath)
