@@ -1,7 +1,9 @@
 __all__ = [
     'configs',
     'event',
+    'mkdir',
     'pgrep',
+    'rmdir',
     'stats',
     'top',
     'tree',
