@@ -30,22 +30,36 @@ from cgutils import command
 from cgutils import host
 from cgutils import formatter
 
+
 class CGTopStats:
     SUBSYSTEMS = ['cpuacct', 'blkio', 'memory']
-    FILTERS   = {
+    FILTERS = {
         'cpuacct': ['stat'],
         'blkio':   ['io_service_bytes'],
         'memory':  ['usage_in_bytes', 'memsw.usage_in_bytes', 'stat'],
-        }
+    }
 
     def __init__(self, options):
         self.options = options
 
+        self.hostcpuinfo = host.CPUInfo()
+
+        self.deltas = {}
+        self.prevs = {}
+
+        self.deltas['cpu'] = 0
+        self.deltas['time'] = 0
+
+        self.cgroups = {}
+        self._update_cgroups()
+        self.last_update_cgroups = time.time()
+
+    def _update_cgroups(self):
         def collect_by_name(cg, store):
-            if cg.name not in store:
-                store[cg.name] = []
-            store[cg.name].append(cg)
-    
+            if cg.fullname not in store:
+                store[cg.fullname] = []
+            store[cg.fullname].append(cg)
+
         # Collect cgroups by group name (path)
         cgroups = {}
         for name in self.SUBSYSTEMS:
@@ -55,16 +69,9 @@ class CGTopStats:
             except EnvironmentError, e:
                 print >> sys.stderr, e
         self.cgroups = cgroups
-        if options.hide_root:
-            del self.cgroups['<root>']
 
-        self.hostcpuinfo = host.CPUInfo()
-
-        self.delta = {}
-        self.prev = {}
-
-        self.delta['cpu'] = 0
-        self.delta['time'] = 0
+        if self.options.hide_root:
+            del self.cgroups['/']
 
     def _get_skelton_stats(self, name, n_procs):
         return {
@@ -77,7 +84,7 @@ class CGTopStats:
             'mem.total': 0,
             'mem.rss': 0,
             'mem.swap':  0,
-            }
+        }
 
     def get_cgroup_stats(self):
         cgroup_stats = []
@@ -98,37 +105,39 @@ class CGTopStats:
             n_procs = len(set(pids))
             if not self.options.show_empty and n_procs == 0:
                 continue
-            
+
             active = False
             stats = self._get_skelton_stats(_cgroup.fullname, n_procs)
 
             if cpu:
                 def percent(delta):
-                    return float(delta)*100/self.delta['cpu']
-                    
-                if self.delta['cpu'] and cpu in self.delta:
-                    stats['cpu.user']   = percent(self.delta[cpu]['user'])
-                    stats['cpu.system'] = percent(self.delta[cpu]['system'])
+                    return float(delta) * 100 / self.deltas['cpu']
+
+                if self.deltas['cpu'] and cpu in self.deltas:
+                    stats['cpu.user'] = percent(self.deltas[cpu]['user'])
+                    stats['cpu.system'] = percent(self.deltas[cpu]['system'])
                 if (stats['cpu.user'] + stats['cpu.system']) > 0.0:
                     active = True
 
             if bio:
                 def byps(delta):
-                    return float(delta)/self.delta['time']
-                if self.delta['time'] and bio in self.delta:
-                    stats['bio.read']  = byps(self.delta[bio]['read'])
-                    stats['bio.write'] = byps(self.delta[bio]['write'])
+                    return float(delta) / self.deltas['time']
+                if self.deltas['time'] and bio in self.deltas:
+                    stats['bio.read'] = byps(self.deltas[bio]['read'])
+                    stats['bio.write'] = byps(self.deltas[bio]['write'])
                 if (stats['bio.read'] + stats['bio.write']) > 0.0:
                     active = True
 
             if mem:
-                if mem in self.delta:
-                    stats['mem.total'] = self.delta[mem]['total']
-                    stats['mem.rss']   = self.delta[mem]['rss']
-                    if 'swap' in self.delta[mem]:
-                        stats['mem.swap']  = self.delta[mem]['swap']
-                if [stats['mem.total'], stats['mem.rss'], \
-                    stats['mem.swap']].count(0) != 3:
+                if mem in self.deltas:
+                    stats['mem.total'] = self.deltas[mem]['total']
+                    stats['mem.rss'] = self.deltas[mem]['rss']
+                    if 'swap' in self.deltas[mem]:
+                        stats['mem.swap'] = self.deltas[mem]['swap']
+                n = [stats['mem.total'],
+                     stats['mem.rss'],
+                     stats['mem.swap']].count(0)
+                if n != 3:
                     active = True
             if not self.options.show_inactive and not active:
                 pass
@@ -139,19 +148,20 @@ class CGTopStats:
     def __conv_blkio_stats(stats):
         n_reads = n_writes = 0L
         for k, v in stats['io_service_bytes'].iteritems():
-            if k == 'Total': continue
+            if k == 'Total':
+                continue
             n_reads += v['Read']
             n_writes += v['Write']
         return {
             'read': n_reads,
             'write': n_writes,
-            }
+        }
 
     def __conv_memory_stats(stats):
         _stats = {}
         _stats['total'] = stats['usage_in_bytes']
         if 'memsw.usage_in_bytes' in stats:
-            _stats['swap']  = stats['memsw.usage_in_bytes'] - _stats['total']
+            _stats['swap'] = stats['memsw.usage_in_bytes'] - _stats['total']
         _stats['rss'] = stats['stat']['rss']
         return _stats
 
@@ -159,13 +169,13 @@ class CGTopStats:
         return {
             'user': stats['stat']['user'],
             'system': stats['stat']['system'],
-            }
+        }
 
     _convert = {
         'memory': __conv_memory_stats,
         'cpuacct': __conv_cpu_stats,
         'blkio': __conv_blkio_stats,
-        }
+    }
 
     def __calc_delta(current, previous):
         delta = {}
@@ -179,16 +189,22 @@ class CGTopStats:
         int: lambda a, b: a - b,
         float: lambda a, b: a - b,
         dict: __calc_delta,
-        }
+    }
 
     def _update_delta(self, key, new):
-        if key in self.prev:
-            self.delta[key] = self._diff[new.__class__](new, self.prev[key])
-        self.prev[key] = new
+        if key in self.prevs:
+            self.deltas[key] = self._diff[new.__class__](new, self.prevs[key])
+        self.prevs[key] = new
 
     def update(self):
-        # Read stats from cgroups and calculate deltas
+        elapsed = time.time() - self.last_update_cgroups
+        if elapsed > self.options.update_cgroups_interval:
+            # Update cgroups hierarchy to know newcomers
+            self._update_cgroups()
+            self.last_update_cgroups = time.time()
+
         removed_group_names = []
+        # Read stats from cgroups and calculate deltas
         for name, cgroup_list in self.cgroups.iteritems():
             try:
                 for _cgroup in cgroup_list:
@@ -212,6 +228,7 @@ class CGTopStats:
         # Calculate delta of time
         self._update_delta('time', time.time())
 
+
 class CGTopUI:
     SORTING_KEYS = [
         'cpu.user',
@@ -269,34 +286,20 @@ class CGTopUI:
             self.options.show_empty = not self.options.show_empty
 
         key_bindings = {
-            ord('q'):
-                lambda: sys.exit(0),
-            ord('Q'):
-                lambda: sys.exit(0),
-            ord('r'):
-                lambda: self.reverse_sorting(),
-            ord('R'):
-                lambda: self.reverse_sorting(),
-            ord('i'):
-                toggle_show_inactive,
-            ord('I'):
-                toggle_show_inactive,
-            ord('z'):
-                toggle_show_zero,
-            ord('Z'):
-                toggle_show_zero,
-            ord('e'):
-                toggle_show_empty,
-            ord('E'):
-                toggle_show_empty,
-            curses.KEY_LEFT:
-                lambda: self.adjust_sorting_key(-1),
-            curses.KEY_RIGHT:
-                lambda: self.adjust_sorting_key(1),
-            curses.KEY_HOME:
-                lambda: self.adjust_sorting_key(-len(self.SORTING_KEYS)),
-            curses.KEY_END:
-                lambda: self.adjust_sorting_key(len(self.SORTING_KEYS)),
+            ord('q'): lambda: sys.exit(0),
+            ord('Q'): lambda: sys.exit(0),
+            ord('r'): lambda: self.reverse_sorting(),
+            ord('R'): lambda: self.reverse_sorting(),
+            ord('i'): toggle_show_inactive,
+            ord('I'): toggle_show_inactive,
+            ord('z'): toggle_show_zero,
+            ord('Z'): toggle_show_zero,
+            ord('e'): toggle_show_empty,
+            ord('E'): toggle_show_empty,
+            curses.KEY_LEFT: lambda: self.adjust_sorting_key(-1),
+            curses.KEY_RIGHT: lambda: self.adjust_sorting_key(1),
+            curses.KEY_HOME: lambda: self.adjust_sorting_key(-len(self.SORTING_KEYS)),
+            curses.KEY_END: lambda: self.adjust_sorting_key(len(self.SORTING_KEYS)),
         }
 
         action = key_bindings.get(key, lambda: None)
@@ -309,19 +312,16 @@ class CGTopUI:
         iterations = 0
         poll = select.poll()
         if not self.options.batch:
-            poll.register(sys.stdin.fileno(), select.POLLIN|select.POLLPRI)
-        while self.options.iterations is None or \
-              iterations < self.options.iterations:
-
-
+            poll.register(sys.stdin.fileno(), select.POLLIN | select.POLLPRI)
+        while self.options.iterations is None or iterations < self.options.iterations:
             bef = time.time()
             self.cgstats.update()
             aft = time.time()
 
-            debug_msg = "%.1f msec to collect statistics" % ((aft-bef)*1000,)
+            debug_msg = "%.1f msec to collect statistics" % ((aft - bef) * 1000)
             self.refresh_display(debug_msg)
 
-            if self.options.iterations is not None:
+            if self.options.iterations:
                 iterations += 1
                 if iterations >= self.options.iterations:
                     break
@@ -343,30 +343,31 @@ class CGTopUI:
 
     def _init_display_params(self):
         subsys_sep_size = 2
-        self.SUBSYS_SEP = ' '*subsys_sep_size
-        item_sep_size   = 1
-        self.ITEM_SEP   = ' '*item_sep_size
+        self.SUBSYS_SEP = ' ' * subsys_sep_size
+        item_sep_size = 1
+        self.ITEM_SEP = ' ' * item_sep_size
         self.ITEM_WIDTHS = {
-            'cpuacct':   formatter.max_width_cpu,
-            'blkio':     formatter.max_width_blkio,
-            'memory':    formatter.max_width_memory,
-            'cpu.user':  formatter.max_width_cpu,
-            'cpu.system':formatter.max_width_cpu,
-            'bio.read':  formatter.max_width_blkio,
-            'bio.write': formatter.max_width_blkio,
-            'mem.total': formatter.max_width_memory,
-            'mem.rss':   formatter.max_width_memory,
-            'mem.swap':  formatter.max_width_memory,
-            'n_procs':   3,
-            'name':      0}
-        self.N_ITEMS = {'cpuacct':2, 'blkio': 2,
+            'cpuacct':    formatter.max_width_cpu,
+            'blkio':      formatter.max_width_blkio,
+            'memory':     formatter.max_width_memory,
+            'cpu.user':   formatter.max_width_cpu,
+            'cpu.system': formatter.max_width_cpu,
+            'bio.read':   formatter.max_width_blkio,
+            'bio.write':  formatter.max_width_blkio,
+            'mem.total':  formatter.max_width_memory,
+            'mem.rss':    formatter.max_width_memory,
+            'mem.swap':   formatter.max_width_memory,
+            'n_procs':    3,
+            'name':       0,
+        }
+        self.N_ITEMS = {'cpuacct': 2, 'blkio': 2,
                         'memory': 3, 'n_procs': 1, 'name': 1}
 
     def _init_subsys_title(self):
         title_list = []
         for name in self.cgstats.SUBSYSTEMS:
-            width = self.ITEM_WIDTHS[name]*self.N_ITEMS[name] + self.N_ITEMS[name] - 1
-            title = '[' + name.upper().center(width-2) + ']'
+            width = self.ITEM_WIDTHS[name] * self.N_ITEMS[name] + self.N_ITEMS[name] - 1
+            title = '[' + name.upper().center(width - 2) + ']'
             title_list.append(title)
         self.SUBSYS_TITLE = self.SUBSYS_SEP.join(title_list)
 
@@ -377,31 +378,31 @@ class CGTopUI:
         titles.append(sep.join([
             'USR'.center(w['cpu.user']),
             'SYS'.center(w['cpu.system']),
-            ]))
+        ]))
         titles.append(sep.join([
             'READ'.center(w['bio.read']),
             'WRITE'.center(w['bio.write']),
-            ]))
+        ]))
         titles.append(sep.join([
             'TOTAL'.center(w['mem.total']),
             'RSS'.center(w['mem.rss']),
             'SWAP'.center(w['mem.swap']),
-            ]))
+        ]))
         titles.append(sep.join([
             '#'.rjust(w['n_procs']),
             'NAME'.rjust(w['name']),
-            ]))
+        ]))
         self.ITEM_TITLE = self.SUBSYS_SEP.join(titles)
         self.KEY2TITLE = {
-            'cpu.user':  'USR',
-            'cpu.system':'SYS',
-            'bio.read':  'READ',
-            'bio.write': 'WRITE',
-            'mem.total': 'TOTAL',
-            'mem.rss':   'RSS',
-            'mem.swap':  'SWAP',
-            'n_procs':   '#',
-            'name':      'NAME',
+            'cpu.user':   'USR',
+            'cpu.system': 'SYS',
+            'bio.read':   'READ',
+            'bio.write':  'WRITE',
+            'mem.total':  'TOTAL',
+            'mem.rss':    'RSS',
+            'mem.swap':   'SWAP',
+            'n_procs':    '#',
+            'name':       'NAME',
         }
 
     def refresh_display(self, debug_msg):
@@ -411,13 +412,13 @@ class CGTopUI:
             strs = []
 
             item2formatters = {
-                'cpu.user':  formatter.percent,
-                'cpu.system':formatter.percent,
-                'bio.read':  formatter.bytepersec,
-                'bio.write': formatter.bytepersec,
-                'mem.total': formatter.byte,
-                'mem.rss':   formatter.byte,
-                'mem.swap':  formatter.byte,
+                'cpu.user':   formatter.percent,
+                'cpu.system': formatter.percent,
+                'bio.read':   formatter.bytepersec,
+                'bio.write':  formatter.bytepersec,
+                'mem.total':  formatter.byte,
+                'mem.rss':    formatter.byte,
+                'mem.swap':   formatter.byte,
             }
 
             def to_s(name):
@@ -432,13 +433,13 @@ class CGTopUI:
             strs.append(sep.join([
                 str(stats['n_procs']).rjust(w['n_procs']),
                 stats['name']]
-                ))
+            ))
             return self.SUBSYS_SEP.join(strs)
 
         cgroup_stats = self.cgstats.get_cgroup_stats()
         cgroup_stats.sort(key=lambda st: st[self.sorting_key],
                           reverse=self.sorting_reverse)
-        lines = [format(s) for s in  cgroup_stats]
+        lines = [format(s) for s in cgroup_stats]
 
         if self.options.batch:
             print debug_msg
@@ -466,7 +467,7 @@ class CGTopUI:
         key_title = self.KEY2TITLE[self.sorting_key]
         pre, post = self.ITEM_TITLE.split(key_title)
         self.win.addstr(pre, curses.A_REVERSE)
-        self.win.addstr(key_title, curses.A_BOLD|curses.A_REVERSE)
+        self.win.addstr(key_title, curses.A_BOLD | curses.A_REVERSE)
         self.win.addstr(post, curses.A_REVERSE)
 
         rest_lines = self.height - n_lines - int(bool(status_msg))
@@ -484,31 +485,36 @@ class CGTopUI:
             self.win.insstr(self.height - 1, 0, status_msg, curses.A_BOLD)
         self.win.refresh()
 
+
 class Command(command.Command):
     NAME = 'top'
 
     parser = command.Command.parser
     parser.add_option('-i', '--show-inactive', action='store_true',
                       dest='show_inactive', default=False,
-                      help='Show inactive groups [False]')
+                      help='Show inactive groups')
     parser.add_option('-z', '--show-zero', action='store_true',
                       dest='show_zero', default=False,
-                      help='Show zero numbers [False]')
+                      help='Show zero numbers')
     parser.add_option('-e', '--show-empty', action='store_true',
                       dest='show_empty', default=False,
-                      help='Hide empty groups [False]')
+                      help='Hide empty groups')
     parser.add_option('-r', '--hide-root', action='store_true',
                       dest='hide_root', default=False,
-                      help='Hide the root group [False]')
+                      help='Hide the root group')
     parser.add_option('-b', '--batch', action='store_true', dest='batch',
                       help='non-interactive mode')
     parser.add_option('-n', '--iter', type='int', dest='iterations',
                       metavar='NUM',
                       help='Number of iterations before ending [infinite]')
     parser.add_option('-d', '--delay', type='float', dest='delay_seconds',
-                      help='Delay between iterations [3 second]',
+                      help='Delay between iterations [%default seconds]',
                       metavar='SEC', default=3)
-
+    parser.add_option('-u', '--update-cgroups-interval', type='float',
+                      dest='update_cgroups_interval',
+                      help='Update cgroups in every this interval [%default seconds]',
+                      metavar='SEC', default=10)
+    parser.usage = "%%prog %s [options]" % NAME
 
     def _run_window(self, win):
         cgstats = CGTopStats(self.options)
@@ -520,4 +526,3 @@ class Command(command.Command):
             return self._run_window(None)
         else:
             return curses.wrapper(self._run_window)
-
